﻿// Library

interface IEdible
{
    public int Energy { get; }
    public void Eat();
}

interface IFruit { }

class Apple : IFruit, IEdible
{
    public int Energy { get; init; }

    public virtual void Eat()
    {
        Console.WriteLine($"Apple eaten, player got {Energy} units of energy.");
    }
}

class GoldenApple : Apple
{
    public override void Eat()
    {
        base.Eat();
        Console.WriteLine("Golden apple increased player's luck.");
    }
}

class Peach : IFruit, IEdible
{
    public int Energy => 8;

    public void Eat()
    {
        Console.WriteLine($"Peach eaten, player got {Energy} units of energy.");
    }
}

class AppleFactory
{
    private int _applesDispensed = 0;
    private int _appleEnergy;

    public AppleFactory(int appleEnergy)
    {
        _appleEnergy = appleEnergy;
    }

    public Apple Create()
    {
        if ((_applesDispensed++) % 2 == 0)
        {
            return new GoldenApple { Energy = _appleEnergy };
        }
        else
        {
            return new Apple { Energy = _appleEnergy };
        }
    }
}

class PeachFactory
{
    public Peach Create() => new Peach();
}

class Tree
{
    private AppleFactory _factory;

    public Tree(AppleFactory factory)
    {
        _factory = factory;
    }

    public List<Apple> Harvest()
    {
        return new List<Apple> {
                _factory.Create(),
                _factory.Create(),
                _factory.Create()
            };
    }
}

// Application

internal class Program
{
    static void Main(string[] args)
    {

        var goodAppleFactory = new AppleFactory(10);
        var goodTree = new Tree(goodAppleFactory);
        HarvestAndEatAllFruits(goodTree);

        var badAppleFactory = new AppleFactory(2);
        var badTree = new Tree(badAppleFactory);
        HarvestAndEatAllFruits(badTree);
    }

    static void HarvestAndEatAllFruits(Tree tree)
    {
        var fruits = tree.Harvest();
        foreach (IFruit fruit in fruits)
        {
            if (fruit is IEdible edible)
            {
                Console.WriteLine("+++ Nice! Got edible fruit, trying to eat it ...");
                edible.Eat();
            }
            else
            {
                Console.WriteLine("+++ Damn! Got inedible fruit.");
            }
        }
        Console.WriteLine();
    }
}
